@@ -34,11 +34,8 @@
 #'
 #' @export
 ImplicationSet <- R6::R6Class(
-
   classname = "ImplicationSet",
-
   public = list(
-
     #' @description
     #' Initialize with an optional name
     #'
@@ -55,7 +52,6 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @return A new \code{ImplicationSet} object.
     initialize = function(...) {
-
       dots <- list(...)
 
       classes <- lapply(dots, function(d) class(d)[1])
@@ -63,7 +59,6 @@ ImplicationSet <- R6::R6Class(
       any_rules <- which(classes == "rules")
 
       if (length(any_rules) > 0) {
-
         arules_imp <- dots[[any_rules[1]]]
 
         attributes <- arules_imp@lhs@itemInfo$labels
@@ -77,14 +72,14 @@ ImplicationSet <- R6::R6Class(
 
         rownames(private$lhs_matrix) <- private$attributes
         rownames(private$rhs_matrix) <- private$attributes
-
       } else {
-
-        args <- list(name = "",
-                     attributes = c(),
-                     lhs = NULL,
-                     rhs = NULL,
-                     I = NULL)
+        args <- list(
+          name = "",
+          attributes = c(),
+          lhs = NULL,
+          rhs = NULL,
+          I = NULL
+        )
         args[names(dots)] <- dots
         private$name <- args$name
         private$attributes <- args$attributes
@@ -92,9 +87,7 @@ ImplicationSet <- R6::R6Class(
         private$rhs_matrix <- args$rhs
         private$I <- args$I
         private$implication_support <- numeric(0)
-
       }
-
     },
 
     #' @description
@@ -104,9 +97,7 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     get_attributes = function() {
-
       return(private$attributes)
-
     },
 
     #' @description
@@ -117,44 +108,38 @@ ImplicationSet <- R6::R6Class(
     #' @return A new \code{ImplicationSet} with only the rules given by the \code{idx} indices (if all \code{idx > 0} and all but \code{idx} if all \code{idx < 0}.
     #' @export
     `[` = function(idx) {
-
       if (is.logical(idx)) {
-
         idx <- which(idx)
-
       }
 
       idx <- idx[abs(idx) <= self$cardinality()]
       idx <- idx[abs(idx) > 0]
 
       if (length(idx) > 0) {
-
         if (all(idx < 0) | all(idx > 0)) {
-
           imp <- ImplicationSet$new(
             name = paste0(private$name, "_", paste0(idx)),
             attributes = private$attributes,
             lhs = Matrix::Matrix(private$lhs_matrix[, idx],
-                                 sparse = TRUE),
+              sparse = TRUE
+            ),
             rhs = Matrix::Matrix(private$rhs_matrix[, idx],
-                                 sparse = TRUE))
+              sparse = TRUE
+            )
+          )
 
           return(imp)
-
         } else {
-
           stop("Cannot use mixed positive and negative indices.\n",
-               call. = FALSE)
-
+            call. = FALSE
+          )
         }
-
       } else {
-
-        return(ImplicationSet$new(attributes = private$attributes,
-                                  I = private$I))
-
+        return(ImplicationSet$new(
+          attributes = private$attributes,
+          I = private$I
+        ))
       }
-
     },
 
     #' @description
@@ -166,30 +151,28 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     to_arules = function(quality = TRUE) {
-
       if (self$is_empty()) {
-
         stop("No implications to export.\n",
-             call. = FALSE)
-
+          call. = FALSE
+        )
       }
 
       if (!private$is_binary()) {
-
         stop("Export to arules format is only allowed when using binary implications.\n",
-             call. = FALSE)
-
+          call. = FALSE
+        )
       }
 
-      rules <- imps_to_arules(LHS = private$lhs_matrix,
-                              RHS = private$rhs_matrix,
-                              attributes = private$attributes,
-                              I = private$I,
-                              quality = quality)
+      rules <- imps_to_arules(
+        LHS = private$lhs_matrix,
+        RHS = private$rhs_matrix,
+        attributes = private$attributes,
+        I = private$I,
+        quality = quality
+      )
 
 
       return(rules)
-
     },
 
     #' @description
@@ -201,59 +184,44 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     add = function(...) {
-
       dots <- list(...)
 
       # Just a single ImplicationSet
       if (length(dots) == 1) {
-
         if (inherits(dots[[1]], "ImplicationSet")) {
-
           private$append_implications(dots[[1]])
-
         } else {
-
           # It is a rules object
           implications <- ImplicationSet$new(dots[[1]])
           private$append_implications(implications)
-
         }
-
       } else {
-
         # It must come in the form LHS, RHS
         if (length(dots) == 2) {
-
           lhs <- dots[[1]]
           rhs <- dots[[2]]
 
           if (inherits(lhs, "Set")) {
-
             lhs <- lhs$get_vector()
-
           }
 
           if (inherits(rhs, "Set")) {
-
             rhs <- rhs$get_vector()
-
           }
 
-          imp <- ImplicationSet$new(attributes = private$attributes,
-                                    lhs = lhs,
-                                    rhs = rhs)
+          imp <- ImplicationSet$new(
+            attributes = private$attributes,
+            lhs = lhs,
+            rhs = rhs
+          )
 
           private$append_implications(imp)
-
         } else {
-
           stop("Invalid number of arguments.\n",
-               call. = FALSE)
-
+            call. = FALSE
+          )
         }
-
       }
-
     },
 
     #' @description
@@ -263,11 +231,11 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     cardinality = function() {
-
-      if (self$is_empty()) return(0)
+      if (self$is_empty()) {
+        return(0)
+      }
 
       ncol(private$lhs_matrix)
-
     },
 
     #' @description
@@ -277,9 +245,7 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     is_empty = function() {
-
       is.null(private$lhs_matrix)
-
     },
 
     #' @description
@@ -289,12 +255,10 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     size = function() {
-
       lhs_size <- Matrix::colSums(private$lhs_matrix)
       rhs_size <- Matrix::colSums(private$rhs_matrix)
 
       return(cbind(LHS = lhs_size, RHS = rhs_size))
-
     },
 
     #' @description
@@ -310,50 +274,63 @@ ImplicationSet <- R6::R6Class(
     closure = function(S,
                        reduce = FALSE,
                        verbose = FALSE) {
-
       if (inherits(S, "Set")) {
-
         original <- S$clone()
         S <- match_attributes(S, private$attributes)
         S <- S$get_vector()
-
       } else {
-
         original <- Set$new(private$attributes, M = S)
-
       }
 
-      cl <- .compute_closure(S,
-                             LHS = private$lhs_matrix,
-                             RHS = private$rhs_matrix,
-                             attributes = private$attributes,
-                             reduce = reduce,
-                             verbose = verbose,
-                             is_direct = private$directness)
+      if (private$hedge == "identity") {
+        cl <- .compute_closure_fuzzy(
+          S,
+          LHS = private$lhs_matrix,
+          RHS = private$rhs_matrix,
+          attributes = private$attributes,
+          reduce = reduce,
+          verbose = verbose,
+          is_direct = private$directness,
+          logic = private$logic
+        )
+      } else { # Should be globalization
+        cl <- .compute_closure(
+          S,
+          LHS = private$lhs_matrix,
+          RHS = private$rhs_matrix,
+          attributes = private$attributes,
+          reduce = reduce,
+          verbose = verbose,
+          is_direct = private$directness
+        )
+      }
 
       if (!reduce) {
-
-        cl <- list(closure = Set$new(attributes = private$attributes,
-                                           M = cl$closure))
-
+        cl <- list(closure = Set$new(
+          attributes = private$attributes,
+          M = cl$closure
+        ))
       } else {
+        cl$closure <- Set$new(
+          attributes = private$attributes,
+          M = cl$closure
+        )
 
-        cl$closure <- Set$new(attributes = private$attributes,
-                                    M = cl$closure)
-
-        cl$implications <- ImplicationSet$new(attributes = private$attributes,
-                                              name = "reduced",
-                                              lhs = cl$implications$lhs,
-                                              rhs = cl$implications$rhs,
-                                              I = private$I)
-
+        cl$implications <- ImplicationSet$new(
+          attributes = private$attributes,
+          name = "reduced",
+          lhs = cl$implications$lhs,
+          rhs = cl$implications$rhs,
+          I = private$I
+        )
       }
 
-      cl$closure <- match_attributes(cl$closure,
-                                     original$get_attributes())
+      cl$closure <- match_attributes(
+        cl$closure,
+        original$get_attributes()
+      )
 
       return(cl)
-
     },
 
     #' @description
@@ -366,19 +343,17 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     recommend = function(S, attribute_filter) {
-
       if (inherits(S, "Set")) {
-
         S <- S$get_vector()
-
       }
 
-      .recommend_attribute(S = S,
-                           LHS = private$lhs_matrix,
-                           RHS = private$rhs_matrix,
-                           attribute_filter = attribute_filter,
-                           attributes = private$attributes)
-
+      .recommend_attribute(
+        S = S,
+        LHS = private$lhs_matrix,
+        RHS = private$rhs_matrix,
+        attribute_filter = attribute_filter,
+        attributes = private$attributes
+      )
     },
 
     #' @description
@@ -398,41 +373,35 @@ ImplicationSet <- R6::R6Class(
     apply_rules = function(rules = c("composition", "generalization"),
                            batch_size = 25000L,
                            parallelize = FALSE,
-                           reorder= FALSE) {
-
+                           reorder = FALSE) {
       # If no implications, do nothing
       if (is.null(private$lhs_matrix) || (ncol(private$lhs_matrix) == 0)) {
-
         return(invisible(self))
-
       }
 
       # If one implication,
       # just "reduction" can be done
       if (ncol(private$lhs_matrix) == 1) {
-
         rules <- intersect(rules, "reduction")
-
       }
 
 
       if (length(rules) > 0) {
-
-        L <- .batch_apply(LHS = private$lhs_matrix,
-                          RHS = private$rhs_matrix,
-                          attributes = private$attributes,
-                          rules = rules,
-                          parallelize = parallelize,
-                          batch_size = batch_size,
-                          reorder = reorder)
+        L <- .batch_apply(
+          LHS = private$lhs_matrix,
+          RHS = private$rhs_matrix,
+          attributes = private$attributes,
+          rules = rules,
+          parallelize = parallelize,
+          batch_size = batch_size,
+          reorder = reorder
+        )
 
         private$lhs_matrix <- L$lhs
         private$rhs_matrix <- L$rhs
-
       }
 
       return(invisible(self))
-
     },
 
     #' @description
@@ -443,7 +412,6 @@ ImplicationSet <- R6::R6Class(
     #' @export
     #'
     to_basis = function() {
-
       LHS <- private$lhs_matrix
       RHS <- private$rhs_matrix
       attributes <- private$attributes
@@ -460,7 +428,97 @@ ImplicationSet <- R6::R6Class(
       private$rhs_matrix <- L$rhs
 
       return(invisible(self))
+    },
 
+    #' @description
+    #' Compute the Direct Optimal Basis using optimized C++ algorithms.
+    #'
+    #' @param method (character) The specific algorithm to run:
+    #' \itemize{
+    #'   \item \code{"direct_optimal"}: (Default) The Direct Optimal Saturation-Pruning algorithm.
+    #'   \item \code{"final_ts"}: Computes Transitive Closure then Prunes (Standard approach).
+    #'   \item \code{"monotonic"}: Incremental algorithm maintaining monotonicity.
+    #'   \item \code{"priority"}: Priority-based refinement algorithm.
+    #' }
+    #' @param verbose (logical) Print verbose output from the C++ backend.
+    #'
+    #' @return Nothing, updates the \code{ImplicationSet} in place with the new basis.
+    #' @export
+    to_direct_optimal = function(method = c("direct_optimal", "final_ts", "monotonic", "priority"),
+                                 verbose = FALSE) {
+      method <- match.arg(method)
+
+      # TODO: What happens with different hedges??!!
+
+      # 1. Prepare Inputs for C++
+      # Calculate L (set of truth degrees present in data + 0 + 1)
+      vals <- unique(c(0, 1, private$lhs_matrix@x, private$rhs_matrix@x))
+      vals <- sort(vals)
+
+      # Retrieve the logic name from the private field
+      # Note: Ensure private$logic holds one of: "Lukasiewicz", "Zadeh", "Godel", "Product"
+      current_logic <- private$logic
+
+      if (is.null(current_logic)) {
+        current_logic <- "Lukasiewicz" # Default fallback
+      }
+
+      # 2. Call C++
+      # Now passing 'current_logic' as an argument
+      res_list <- switch(method,
+        "direct_optimal" = run_direct_optimal_sp_rcpp_optimized(
+          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+        ),
+        "final_ts" = run_final_ts_rcpp_optimized(
+          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+        ),
+        "monotonic" = run_monotonic_incremental_rcpp_optimized(
+          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+        ),
+        "priority" = run_priority_refinement_rcpp_optimized(
+          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+        )
+      )
+
+      # 3. Reconstruct Sparse Matrices
+      s_data <- res_list$Sigma
+      n_rules <- s_data$n_rules
+      n_attrs <- length(private$attributes)
+
+      if (n_rules > 0) {
+        # Convert 0-based indices (C++) to 1-based indices (R)
+        new_lhs <- Matrix::sparseMatrix(
+          i = s_data$lhs_i + 1,
+          j = s_data$lhs_j + 1,
+          x = s_data$lhs_x,
+          dims = c(n_attrs, n_rules),
+          dimnames = list(private$attributes, NULL)
+        )
+
+        new_rhs <- Matrix::sparseMatrix(
+          i = s_data$rhs_i + 1,
+          j = s_data$rhs_j + 1,
+          x = s_data$rhs_x,
+          dims = c(n_attrs, n_rules),
+          dimnames = list(private$attributes, NULL)
+        )
+      } else {
+        new_lhs <- Matrix::Matrix(0, nrow = n_attrs, ncol = 0, sparse = TRUE)
+        new_rhs <- Matrix::Matrix(0, nrow = n_attrs, ncol = 0, sparse = TRUE)
+      }
+
+      # 4. Update Internal State
+      private$lhs_matrix <- new_lhs
+      private$rhs_matrix <- new_rhs
+      private$implication_support <- numeric(0)
+      private$directness <- TRUE
+
+      if (verbose) {
+        cat("Algorithm finished using logic:", current_logic, "\n")
+        print(res_list$metrics)
+      }
+
+      return(invisible(self))
     },
 
     #' @description
@@ -470,35 +528,35 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     print = function() {
-
       if (is.null(private$lhs_matrix)) {
-
         cat("Implication set with 0 implications.\n")
 
         return(invisible(self))
-
       }
 
       n_implications <- ncol(private$lhs_matrix)
       cat("Implication set with", n_implications, "implications.\n")
 
       if (n_implications > 0) {
-
         attributes <- private$attributes
         LHS <- private$lhs_matrix
         RHS <- private$rhs_matrix
 
-        implications <- sapply(seq(n_implications),
-                               function(i) paste0("Rule ", i, ": ",
-                               # function(i) paste0("Rule: ",
-                                                  .implication_to_string(LHS[, i], RHS[, i], attributes)))
+        implications <- sapply(
+          seq(n_implications),
+          function(i) {
+            paste0(
+              "Rule ", i, ": ",
+              # function(i) paste0("Rule: ",
+              .implication_to_string(LHS[, i], RHS[, i], attributes)
+            )
+          }
+        )
 
         implications <- sapply(implications, function(s) stringr::str_wrap(s, width = getOption("width"), exdent = 2))
 
         cat(implications, sep = "\n")
-
       }
-
     },
 
     #' @description
@@ -516,20 +574,17 @@ ImplicationSet <- R6::R6Class(
                         ncols = 1,
                         numbered = TRUE,
                         numbers = seq(self$cardinality())) {
-
       output <- imp_to_latex(self,
-                             ncols = ncols,
-                             numbered = numbered,
-                             numbers = numbers)
+        ncols = ncols,
+        numbered = numbered,
+        numbers = numbers
+      )
 
       if (print) {
-
         cat(output)
-
       }
 
       return(invisible(output))
-
     },
 
     #' @description
@@ -539,25 +594,22 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     get_LHS_matrix = function() {
-
       if (self$is_empty()) {
-
         LHS <- Matrix::Matrix(FALSE,
-                      nrow = length(private$attributes),
-                      ncol = 1,
-                      sparse = TRUE)
-
+          nrow = length(private$attributes),
+          ncol = 1,
+          sparse = TRUE
+        )
       } else {
-
         LHS <- private$lhs_matrix
-
       }
 
-      dimnames(LHS) <- list(private$attributes,
-                            paste0(seq(ncol(LHS))))
+      dimnames(LHS) <- list(
+        private$attributes,
+        paste0(seq(ncol(LHS)))
+      )
 
       return(LHS)
-
     },
 
     #' @description
@@ -567,26 +619,23 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     get_RHS_matrix = function() {
-
       if (self$is_empty()) {
-
         RHS <- Matrix::Matrix(FALSE,
-                      nrow = length(private$attributes),
-                      ncol = 1,
-                      sparse = TRUE)
-
+          nrow = length(private$attributes),
+          ncol = 1,
+          sparse = TRUE
+        )
       } else {
-
         RHS <- private$rhs_matrix
-
       }
 
-      dimnames(RHS) <- list(private$attributes,
-                            paste0(seq(ncol(RHS))))
+      dimnames(RHS) <- list(
+        private$attributes,
+        paste0(seq(ncol(RHS)))
+      )
 
 
       return(RHS)
-
     },
 
 
@@ -607,110 +656,85 @@ ImplicationSet <- R6::R6Class(
                       rhs = NULL,
                       not_rhs = NULL,
                       drop = FALSE) {
-
       RHS <- private$rhs_matrix
       LHS <- private$lhs_matrix
 
       if (!is.null(lhs)) {
-
         # Filter the implications which have
         # the given lhs
-        idx_attr <- match(lhs,
-                          private$attributes)
+        idx_attr <- match(
+          lhs,
+          private$attributes
+        )
 
         if (length(idx_attr) > 1) {
-
           idx_lhs <- Matrix::which(Matrix::colSums(LHS[idx_attr, ]) > 0)
-
         } else {
-
           idx_lhs <- which(LHS[idx_attr, ] > 0)
-
         }
-
       } else {
-
         # If not specified a filter for LHS,
         # select all implications
         idx_lhs <- seq(ncol(LHS))
-
       }
 
       if (!is.null(not_lhs)) {
-
         # Filter the implications which
         # does not have the given not_lhs
-        idx_attr <- match(not_lhs,
-                          private$attributes)
+        idx_attr <- match(
+          not_lhs,
+          private$attributes
+        )
 
         if (length(idx_attr) > 1) {
-
           idx_not_lhs <- Matrix::which(Matrix::colSums(LHS[idx_attr, ]) > 0)
-
         } else {
-
           idx_not_lhs <- which(LHS[idx_attr, ] > 0)
-
         }
-
       } else {
-
         # If not specified a filter for LHS,
         # select none.
         idx_not_lhs <- c()
-
       }
 
       idx_lhs <- setdiff(idx_lhs, idx_not_lhs)
 
       if (!is.null(rhs)) {
-
         # Filter the implications which have
         # the given lhs
-        idx_attr <- match(rhs,
-                          private$attributes)
+        idx_attr <- match(
+          rhs,
+          private$attributes
+        )
 
         if (length(idx_attr) > 1) {
-
           idx_rhs <- Matrix::which(Matrix::colSums(RHS[idx_attr, ]) > 0)
-
         } else {
-
           idx_rhs <- which(RHS[idx_attr, ] > 0)
-
         }
-
       } else {
-
         # If not specified a filter for RHS,
         # select all implications
         idx_rhs <- seq(ncol(RHS))
-
       }
 
       if (!is.null(not_rhs)) {
-
         # Filter the implications which
         # does not have the given not_lhs
-        idx_attr <- match(not_rhs,
-                          private$attributes)
+        idx_attr <- match(
+          not_rhs,
+          private$attributes
+        )
 
         if (length(idx_attr) > 1) {
-
           idx_not_rhs <- Matrix::which(Matrix::colSums(RHS[idx_attr, ]) > 0)
-
         } else {
-
           idx_not_rhs <- which(RHS[idx_attr, ] > 0)
-
         }
-
       } else {
-
         # If not specified a filter for LHS,
         # select none.
         idx_not_rhs <- c()
-
       }
 
       idx_rhs <- setdiff(idx_rhs, idx_not_rhs)
@@ -718,42 +742,38 @@ ImplicationSet <- R6::R6Class(
       idx <- intersect(idx_lhs, idx_rhs)
 
       if (length(idx) == 0) {
-
         warning("No combination of given LHS and RHS found.\n",
-                call. = FALSE,
-                immediate. = TRUE)
+          call. = FALSE,
+          immediate. = TRUE
+        )
         return(invisible(NULL))
-
       }
 
       if (length(idx) > 0) {
-
         if (drop && !is.null(rhs)) {
-
           newLHS <- LHS[, idx]
           newRHS <- RHS[, idx]
 
           other_idx <- setdiff(seq(nrow(RHS)), idx_attr)
           newRHS[other_idx, ] <- 0
 
-          imp <- ImplicationSet$new(name = paste0(private$name, "_filtered"),
-                                    attributes = private$attributes,
-                                    lhs = Matrix::Matrix(newLHS, sparse = TRUE),
-                                    rhs = Matrix::Matrix(newRHS, sparse = TRUE))
-
+          imp <- ImplicationSet$new(
+            name = paste0(private$name, "_filtered"),
+            attributes = private$attributes,
+            lhs = Matrix::Matrix(newLHS, sparse = TRUE),
+            rhs = Matrix::Matrix(newRHS, sparse = TRUE)
+          )
         } else {
-
-          imp <- ImplicationSet$new(name = paste0(private$name, "_filtered"),
-                                    attributes = private$attributes,
-                                    lhs = Matrix::Matrix(LHS[, idx], sparse = TRUE),
-                                    rhs = Matrix::Matrix(RHS[, idx], sparse = TRUE))
-
+          imp <- ImplicationSet$new(
+            name = paste0(private$name, "_filtered"),
+            attributes = private$attributes,
+            lhs = Matrix::Matrix(LHS[, idx], sparse = TRUE),
+            rhs = Matrix::Matrix(RHS[, idx], sparse = TRUE)
+          )
         }
 
         return(imp)
-
       }
-
     },
 
     #' @description
@@ -762,95 +782,121 @@ ImplicationSet <- R6::R6Class(
     #' @return A vector with the support of each implication
     #' @export
     support = function() {
-
       if (self$is_empty()) {
-
         return(numeric(0))
-
       }
 
       if (length(private$implication_support) > 0) {
-
         return(private$implication_support)
-
       }
 
-      subsets <- .subset(private$lhs_matrix,
-                         private$I)
+      subsets <- .subset(
+        private$lhs_matrix,
+        private$I
+      )
 
       private$implication_support <- Matrix::rowMeans(subsets)
 
       return(private$implication_support)
+    },
 
+    #' @description
+    #' Sets the logic to use
+    #'
+    #'
+    #' @param name The name of the logic to use. To see the available names, run \code{available_logics()}.
+    #'
+    #' @export
+    use_logic = function(name = available_logics()) {
+      name <- match.arg(name)
+      if (name %in% available_logics()) {
+        private$logic <- name
+      }
+    },
+
+    #' @description
+    #' Gets the logic used
+    #'
+    #' @return A string with the name of the logic.
+    #'
+    #' @export
+    get_logic = function() {
+      private$logic
+    },
+
+    #' @description
+    #' Sets the hedge to use when computing closures
+    #'
+    #'
+    #' @param name The name of the hedge to use. Only "globalization" and "identity" are allowed.
+    #'
+    #' @export
+    use_hedge = function(name = c("globalization", "identity")) {
+      name <- match.arg(name)
+      if (name %in% c("globalization", "identity")) {
+        private$hedge <- name
+      } else {
+        warning("No hedge with that name found.")
+      }
+    },
+
+    #' @description
+    #' Gets the hedge used to compute closures
+    #'
+    #' @return A string with the name of the hedge
+    #'
+    #' @export
+    get_hedge = function() {
+      private$hedge
     }
-
   ),
-
   private = list(
-
     name = "",
-
     attributes = NULL,
-
     lhs_matrix = NULL,
     rhs_matrix = NULL,
-
     I = NULL,
     implication_support = NULL,
     binary = NULL,
     directness = FALSE,
-
+    logic = "Godel",
+    hedge = "globalization",
     is_binary = function() {
-
       if (!is.null(private$binary)) {
-
         return(private$binary)
-
       }
 
       if (!is.null(private$I)) {
-
         v <- unique(c(0, private$I@x, 1))
-
       } else {
-
         if (!is.null(private$lhs_matrix)) {
-
-          v <- unique(c(0, private$lhs_matrix@x,
-                        private$rhs_matrix@x, 1))
-
+          v <- unique(c(
+            0, private$lhs_matrix@x,
+            private$rhs_matrix@x, 1
+          ))
         } else {
-
           return(FALSE)
-
         }
-
       }
 
       private$binary <- (length(v) == 2) && all(v == c(0, 1))
       return(private$binary)
-
-
     },
-
     append_implications = function(implications) {
+      imps <- match_implications(
+        implications,
+        private$attributes
+      )
 
-      LHS <- implications$get_LHS_matrix()
-      RHS <- implications$get_RHS_matrix()
+      LHS <- imps$get_LHS_matrix()
+      RHS <- imps$get_RHS_matrix()
 
       if (length(private$attributes) == nrow(LHS)) {
-
         private$lhs_matrix <- cbind(private$lhs_matrix, LHS)
         private$rhs_matrix <- cbind(private$rhs_matrix, RHS)
-
       } else {
-
         stop("Dimensions mismatch.")
-
       }
-
     }
-
   )
-
 )
